package com.huiju.welcome.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.welcome.mapper.TaCustomerMapMapper;
import com.huiju.welcome.mapper.TaCustomerMapper;
import com.huiju.welcome.mapper.TaMainUsherRecordMapper;
import com.huiju.welcome.mapper.TaWechatUserMapper;
import com.huiju.welcome.model.*;
import com.huiju.welcome.service.ITaCustomerService;
import com.huiju.welcome.service.ITaMainUsherRecordService;
import com.huiju.welcome.service.ITaPersonService;
import com.huiju.welcome.utils.CommConstant;
import com.huiju.welcome.utils.MiniApp;
import com.huiju.welcome.utils.StatusUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-06-12
 */
@Service
public class TaCustomerServiceImpl extends ServiceImpl<TaCustomerMapper, TaCustomer> implements ITaCustomerService {
    Logger log = LoggerFactory.getLogger(TaCustomerServiceImpl.class);

    @Autowired
    TaCustomerMapper taCustomerMapper;

    @Autowired
    TaCustomerMapMapper taCustomerMapMapper;

    @Autowired
    TaMainUsherRecordMapper taMainUsherRecordMapper;

    @Autowired
    ITaPersonService iTaPersonService;

    @Autowired
    ITaMainUsherRecordService iTaMainUsherRecordService;

    @Autowired
    TaWechatUserMapper taWechatUserMapper;

    @Autowired
    MiniApp miniApp;

    @Override
    public TaCustomer newVisitRecord(TaMainUsherRecord taMainUsherRecord) throws Exception {
        Integer personId = taMainUsherRecord.getPersonId();
        if (null == personId) {
            TaPerson taPerson = iTaPersonService.newPersonByUsher(taMainUsherRecord);
            personId = taPerson.getRealId();
            taMainUsherRecord.setPersonId(personId);
        }

        Integer custId = taMainUsherRecord.getCustomerId();
        TaCustomer cust;
        if (null == custId) {
            cust = newCustomerFromMainUsher(taMainUsherRecord);
            if (null == cust) {
                throw new Exception("生成置业客户失败");
            }
            taMainUsherRecord.setCustomerId(cust.getCustomerId());
        } else {
            cust = taCustomerMapper.selectById(custId);
        }

        TaCustomerMap taCustomerMap = taCustomerMapMapper.getCustomerOf(personId);
        if (null == taCustomerMap) {
            TaCustomerMap custMap = new TaCustomerMap();
            custMap.setCustomerId(cust.getCustomerId());
            custMap.setPersonId(taMainUsherRecord.getPersonId());

            int row = taCustomerMapMapper.insert(custMap);
            if (row <= 0) {
                throw new Exception("生成置业客户映射失败");
            }
        }

        return cust;
    }

    @Override
    public List<TaPerson> getGroup(Integer customerId) {
        return taCustomerMapMapper.getGroup(customerId);
    }

    @Override
    public Map<String, Object> getLastVisitInfo(Integer customerId) {
        return taMainUsherRecordMapper.getSummaryOfCustomer(customerId);
    }

    @Override
    public void judgeNewCust(TaPerson taPerson) throws Exception {
        // 首先判定这个人 2 小时内是否来访过
        LocalDateTime dt = LocalDateTime.now().minusHours(2);
        List<TaMainUsherRecord> lastList = taMainUsherRecordMapper.getLastList(taPerson.getPersonId(), dt);
        if (null != lastList && lastList.size() > 0) return;

        LocalDateTime now = LocalDateTime.now();

        // 查询是否有映射客户 - 如果有多个映射客户, 先随机取一条
        // 暂时没有找到好的替代方案
        TaCustomer taCustomer = taCustomerMapper.getCustomerOf(taPerson.getPersonId(), taPerson.getRealId());

        // 如果是新人过来
        if (null == taCustomer) {
            taCustomer = newCustomerByPerson(taPerson);
        }

        // 新增前台记录
        iTaMainUsherRecordService.newRecordByDevice(taCustomer);

        return;
    }

    @Override
    public void deleteCustOfPerson(Integer personId, Integer realId) {
        // 删除映射表
        QueryWrapper<TaCustomerMap> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("person_id", personId).or().eq("person_id", realId);
        taCustomerMapMapper.delete(queryWrapper);

        // 删除客户表, 更新状态为删除
        // 默认客户ID 与人员ID是一致的
        UpdateWrapper<TaCustomer> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("status", StatusUtils.Delete);
        updateWrapper.eq("customer_id", personId).or().eq("customer_id", realId);
        taCustomerMapper.update(new TaCustomer(), updateWrapper);
    }

    @Override
    public List<TaCustomer> getSomeCustMayBe(String plateNumber) {
        return taCustomerMapper.getSomeCustMayBe(plateNumber);
    }

    @Override
    public TaCustomer newCustByCarDevice(TaCarRecord taCarRecord) {
        TaCustomer taCustomer = new TaCustomer();

        taCustomer.setCreateDate(LocalDateTime.now());
        taCustomer.setStatus(StatusUtils.Normal);
        taCustomer.setCustomerName(taCarRecord.getCustomerName());
        taCustomer.setAvatar(taCarRecord.getDriverImage());
        taCustomer.setCarModel(taCarRecord.getCarModel());
        taCustomer.setPlateNumber(taCarRecord.getPlateNumber());

        TaPerson taPerson = iTaPersonService.newPersonByFront(taCustomer);
        taCustomer.setCustomerId(taPerson.getRealId());

        TaCustomerMap customerMap = new TaCustomerMap();
        customerMap.setCustomerId(taCustomer.getCustomerId());
        customerMap.setPersonId(taPerson.getPersonId());

        taCustomerMapMapper.insert(customerMap);
        taCustomerMapper.insert(taCustomer);

        return taCustomer;
    }

    @Override
    public void notifyConsultant(TaCustomer taCustomer) throws Exception {
        String tpl = miniApp.getTpls().getVisitor();
        String page = "";
        String openid = taWechatUserMapper.getOpenidOf(taCustomer.getConsultantId(), CommConstant.WECHAT_CLIENT_MINIAPP);
        if (null == openid) {
            throw new Exception("发送消息给置业失败, 当前置业[" + taCustomer.getConsultantId() + "]未绑定小程序账号");
        }
        List<Object> data = new ArrayList<Object>(){{
            add(taCustomer.getCustomerName());
            add(taCustomer.getPhone());
            add("您有客户来访, 请及时接待!");
        }};

        miniApp.sendTPLMessage(tpl, openid, page, data);
    }

    private TaCustomer newCustomerByPerson(TaPerson taPerson) throws Exception {
        TaCustomer taCustomer = new TaCustomer();
        taCustomer.setCustomerId(taPerson.getRealId());
        taCustomer.setAvatar(taPerson.getAvatar());
        taCustomer.setPhone(taPerson.getPhone());
//            taCustomer.setSex(taPerson.getSex());
        taCustomer.setCustomerName(taPerson.getName());
        taCustomer.setCreateDate(LocalDateTime.now());
        taCustomer.setStatus(StatusUtils.Normal);
        int row = taCustomerMapper.insert(taCustomer);

        if (row < 1) {
            throw new Exception("新增客户信息失败");
        }

        int mapsNum = taPerson.getPersonId().equals(taPerson.getRealId()) ? 1 : 2;

        for (int i = 0; i < mapsNum; i ++) {
            TaCustomerMap custMap = new TaCustomerMap();
            custMap.setPersonId(i == 0 ? taPerson.getRealId() : taPerson.getPersonId());
            custMap.setCustomerId(taCustomer.getCustomerId());
            row = taCustomerMapMapper.insert(custMap);
            if (row < 1) {
                throw new Exception("新增客户映射信息失败");
            }
        }

        return taCustomer;
    }

    private TaCustomer newCustomerFromMainUsher(TaMainUsherRecord taMainUsherRecord) {
        TaCustomer taCustomer = new TaCustomer();
        taCustomer.setCustomerId(taMainUsherRecord.getPersonId());       // 默认客户ID与人员ID一致
        taCustomer.setCustomerName(taMainUsherRecord.getPersonName());
        taCustomer.setSex(taMainUsherRecord.getSex());
        taCustomer.setPhone(taMainUsherRecord.getPhone());
        taCustomer.setAvatar(taMainUsherRecord.getAvatar());
        taCustomer.setPlateNumber(taMainUsherRecord.getPlateNumber());
        taCustomer.setCarModel(taMainUsherRecord.getCarModel());
        taCustomer.setChannel(taMainUsherRecord.getChannel());
        taCustomer.setPurpose(taMainUsherRecord.getPurpose());
        taCustomer.setConsultantId(taMainUsherRecord.getConsultantId());
        taCustomer.setRealtyConsultant(taMainUsherRecord.getRealtyConsultant());
        taCustomer.setStatus(StatusUtils.Normal);
        taCustomer.setCreateDate(LocalDateTime.now());

        int row = taCustomerMapper.insert(taCustomer);
        if (row < 1) return null;

        return taCustomer;
    }

}
