package com.huiju.welcome.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.welcome.common.Action;
import com.huiju.welcome.common.WebSocketServer;
import com.huiju.welcome.mapper.SysUserMapper;
import com.huiju.welcome.mapper.TaMainUsherRecordMapper;
import com.huiju.welcome.mapper.TaVisitorAppointmentMapper;
import com.huiju.welcome.mapper.TaWechatUserMapper;
import com.huiju.welcome.model.*;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.mapper.TaCustomerMapper;
import com.huiju.welcome.model.TaCustomer;
import com.huiju.welcome.model.TaMainUsherRecord;
import com.huiju.welcome.service.ITaMainUsherRecordService;
import com.huiju.welcome.utils.CommConstant;
import com.huiju.welcome.utils.MiniApp;
import com.huiju.welcome.utils.StatusUtils;
import com.huiju.welcome.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-06-04
 */
@Service
public class TaMainUsherRecordServiceImpl extends ServiceImpl<TaMainUsherRecordMapper, TaMainUsherRecord> implements ITaMainUsherRecordService {
    @Autowired
    private TaMainUsherRecordMapper taMainUsherRecordMapper;

    @Autowired
    private TaCustomerMapper taCustomerMapper;

    @Autowired
    TaVisitorAppointmentMapper taVisitorAppointmentMapper;

    @Autowired
    TaWechatUserMapper taWechatUserMapper;

    @Autowired
    SysUserMapper sysUserMapper;

    @Autowired
    MiniApp miniApp;

    @Override
    public boolean updateNormal(List<TaMainUsherRecord> records) {
        if (records == null || records.size() == 0) return true;

        List<Integer> recs = new ArrayList<>();
        for (TaMainUsherRecord rec: records) {
            recs.add(rec.getRecId());
        }

        UpdateWrapper<TaMainUsherRecord> wrapper = new UpdateWrapper();
        wrapper.set("status", StatusUtils.Normal);
        wrapper.in("rec_id", recs);

        int row = taMainUsherRecordMapper.update(new TaMainUsherRecord(), wrapper);
        return row >= 0;
    }

    @Override
    public boolean newRecordByDevice(TaCustomer taCustomer) {
        TaMainUsherRecord taMainUsherRecord = newRecordByCust(taCustomer);
        int row = taMainUsherRecordMapper.insert(taMainUsherRecord);

        try {
            Action act = Action.create(Action.TYPE_NEW_MAIN_USHER, JSONObject.toJSONString(taMainUsherRecord));
            WebSocketServer.sendInfo(act.toJSON(), "miniapp", null);
        } catch (Exception e) {}

        // 通知对应的置业
        if (null != taCustomer.getConsultantId()) {
            try {
                notifyConsultant(taMainUsherRecord, taCustomer);
            } catch (Exception e) {}
        }

        return row > 0;
    }

    @Override
    public boolean newRecordByConsultant(TaCustomer taCustomer) {
        TaMainUsherRecord taMainUsherRecord = newRecordByCust(taCustomer);
        int row = taMainUsherRecordMapper.insert(taMainUsherRecord);

        return row > 0;
    }

    private TaMainUsherRecord newRecordByCust(TaCustomer taCustomer) {
        TaMainUsherRecord taMainUsherRecord = new TaMainUsherRecord();
        taMainUsherRecord.setCustomerId(taCustomer.getCustomerId());
        taMainUsherRecord.setAppointment(taCustomer.getApointment());
        taMainUsherRecord.setConsultantId(taCustomer.getConsultantId());
        taMainUsherRecord.setChannel(taCustomer.getChannel());
        taMainUsherRecord.setPurpose(taCustomer.getPurpose());
        taMainUsherRecord.setPlateNumber(taCustomer.getPlateNumber());
        taMainUsherRecord.setVisiteDate(LocalDateTime.now());
        taMainUsherRecord.setStatus(StatusUtils.Normal);
        return taMainUsherRecord;
    }

    @Override
    public IPage<TaMainUsherRecord> taMainUsherRecordList(IPage<TaMainUsherRecord> pg,String phone,String platNumber,String personName,Integer appointment) {
        IPage<TaMainUsherRecord>  taMainUsherRecordList= taMainUsherRecordMapper.taMainUsherRecordList(pg,phone,platNumber,personName,appointment);
        return taMainUsherRecordList;
    }

    @Override
    public void notifyConsultant(TaMainUsherRecord taMainUsherRecord, TaCustomer taCustomer) throws Exception {
        Integer realConsultant = taCustomer.getConsultantId();
        Integer proxyConsultant = taMainUsherRecord.getConsultantId();

        // 查找置业微信ID
        String openid = taWechatUserMapper.getOpenidOf(proxyConsultant, CommConstant.WECHAT_CLIENT_MINIAPP);
        if (StringUtils.isEmpty(openid)) {
            throw new Exception("置业尚未绑定微信小程序");
        };

        String visitorName = taMainUsherRecord.getPersonName();
        String phone = taMainUsherRecord.getPhone();
        Integer customerId = taMainUsherRecord.getCustomerId();

        String page = CommConstant.PAGE_MY_CUSTOMER + "?id=" + String.valueOf(customerId);
        String tplId = miniApp.getTpls().getVisitor();

        // 如果是客户对应的置业进行接待
        if (realConsultant.equals(proxyConsultant)) {
            List<Object> data = new ArrayList<Object>(){{
                add(visitorName);
                add(phone);
                add("您有新客户来访, 请及时接待!");
            }};

            miniApp.sendTPLMessage(tplId, openid, page, data);
        } else {
            // 否则, 给原置业发消息, 给接待置业发消息

            // 原绑定置业消息
            String rcOpenid = taWechatUserMapper.getOpenidOf(realConsultant, CommConstant.WECHAT_CLIENT_MINIAPP);
            if (StringUtils.isEmpty(openid)) {
                throw new Exception("置业尚未绑定微信小程序");
            };

            SysUser usr1 = sysUserMapper.selectById(proxyConsultant);
            List<Object> dt1 = new ArrayList<Object>(){{
                add(visitorName);
                add(phone);
                add("您有新客户来访, 已被安排给 " + usr1.getUsername() + " 接待!");
            }};
            miniApp.sendTPLMessage(tplId, rcOpenid, page, dt1);



            // 代理置业
            SysUser usr2 = sysUserMapper.selectById(realConsultant);
            List<Object> dt2 = new ArrayList<Object>(){{
                add(visitorName);
                add(phone);
                add(usr2.getUsername() + " 有新客户来访, 请协助接待一下!");
            }};
            miniApp.sendTPLMessage(tplId, openid, page, dt2);
        }
    }
    public ResponseBean customerPage(int pageNum, int pageSize, String customerName, String phone, String platNumber) {
        ResponseBean response = new ResponseBean();
        IPage<TaCustomer> pg = new Page<>(pageNum, pageSize);
        IPage<TaCustomer> result = taCustomerMapper.customerPage(pg,customerName,phone,platNumber);
        response.addSuccess(result);
        return response;
    }
}
